'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.caches = exports.stores = exports.default = undefined;

var _class, _temp; /* eslint-disable class-methods-use-this */


var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _Memory = require('./cache/Memory');

var _Memory2 = _interopRequireDefault(_Memory);

var _stores = require('./stores');

var stores = _interopRequireWildcard(_stores);

var _cache = require('./cache');

var caches = _interopRequireWildcard(_cache);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

let Config = (_temp = _class = class Config {

  constructor(cache = new _Memory2.default()) {
    this.stores = [];
    this.cache = cache;
  }

  addStore(store) {
    const nextStore = this.stores[0] || {
      get: () => this.constructor.notFoundValue
    };

    this.stores.unshift({
      get: key => store.get(key, nextStore)
    });
  }

  multiGet(keys, ttl) {
    var _this = this;

    return _asyncToGenerator(function* () {
      (0, _invariant2.default)(Array.isArray(keys), 'Config.multiGet requires an array of keys!');
      return Promise.all(keys.map(function (key) {
        return _this.get(key, undefined, ttl);
      }));
    })();
  }

  get(key, defaultValue, ttl) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      (0, _invariant2.default)(key, 'Key is required!');

      if (_this2.cache.has(key)) {
        return _this2.cache.get(key);
      }

      const value = yield _this2.stores[0].get(key);

      if (value === _this2.constructor.notFoundValue) {
        return defaultValue;
      }

      _this2.cache.set(key, value, ttl);

      return value;
    })();
  }

  clearCache(key) {
    if (key) {
      this.cache.remove(key);
    } else {
      this.cache.clear();
    }
  }
}, _class.notFoundValue = () => {}, _temp);
exports.default = Config;
exports.stores = stores;
exports.caches = caches;