'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _set = require('lodash/set');

var _set2 = _interopRequireDefault(_set);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

let AliasStore = class AliasStore {
  constructor(backend, aliases = {}) {
    this.backend = backend;
    this.aliases = aliases;
  }

  add(from, to) {
    if (typeof from === 'string') {
      this.aliases[from] = to;
    } else {
      this.aliases = _extends({}, this.aliases, from);
    }

    return this;
  }

  hasAlias(key) {
    return Object.keys(this.aliases).includes(key);
  }

  embedsAlias(key) {
    const regex = new RegExp(`^${key}\\.`);
    return Object.keys(this.aliases).find(alias => regex.test(alias));
  }

  getNestedKeys(key) {
    const regex = new RegExp(`^${key}\\.`);
    return Object.keys(this.aliases).filter(alias => regex.test(alias));
  }

  has(key) {
    return this.hasAlias(key) || this.embedsAlias(key);
  }

  getBackendValue(key) {
    return typeof key === 'string' ? this.backend.get(key) : key(this.backend);
  }

  get(key, nextStore) {
    var _this = this;

    return _asyncToGenerator(function* () {
      if (!_this.has(key)) {
        return nextStore.get(key);
      }

      if (_this.hasAlias(key)) {
        return _this.getBackendValue(_this.aliases[key]);
      }

      const value = yield nextStore.get(key);

      const nestedKeys = _this.getNestedKeys(key);

      if (nestedKeys.length) {
        yield Promise.all(nestedKeys.map((() => {
          var _ref = _asyncToGenerator(function* (nestedKey) {
            const val = yield _this.getBackendValue(nestedKey);
            (0, _set2.default)(value, nestedKey.substr(key.length + 1), val);
          });

          return function (_x) {
            return _ref.apply(this, arguments);
          };
        })()));
      }

      return value;
    })();
  }
};
exports.default = AliasStore;